# coding=utf-8
import datetime
import logging
import re
import time
from logging.handlers import TimedRotatingFileHandler


class LoggerUtils(object):

    def __init__(self, filename):
        logging.basicConfig(level=logging.INFO)
        self.logger = logging.getLogger(filename)
        self.logger.propagate = False
        log_fmt = '%(asctime)s %(filename)s[line:%(lineno)d] %(levelname)s %(message)s'
        formatter = logging.Formatter(log_fmt)
        log_file_handler = TimedRotatingFileHandler(
            filename='./logs/%s.log' % filename, when="H", interval=1,
            backupCount=24)
        log_file_handler.suffix = "%Y-%m-%d_%H.log"
        log_file_handler.extMatch = re.compile(r"^\d{4}-\d{2}-\d{2}_\d{2}-\d{2}.log$")
        log_file_handler.setFormatter(formatter)
        log_file_handler.setLevel(logging.DEBUG)
        self.logger.handlers = []
        self.logger.addHandler(log_file_handler)


class Formatter(logging.Formatter):
    def formatTime(self, record, datefmt=None):
        """
        Return the creation time of the specified LogRecord as formatted text.

        This method should be called from format() by a formatter which
        wants to make use of a formatted time. This method can be overridden
        in formatters to provide for any specific requirement, but the
        basic behaviour is as follows: if datefmt (a string) is specified,
        it is used with time.strftime() to format the creation time of the
        record. Otherwise, the ISO8601 format is used. The resulting
        string is returned. This function uses a user-configurable function
        to convert the creation time to a tuple. By default, time.localtime()
        is used; to change this for a particular formatter instance, set the
        'converter' attribute to a function with the same signature as
        time.localtime() or time.gmtime(). To change it for all formatters,
        for example if you want all logging times to be shown in GMT,
        set the 'converter' attribute in the Formatter class.
        """
        ct = self.converter(record.created)
        if datefmt:
            s = time.strftime(datefmt, ct)
        else:
            # t = time.strftime("%Y-%m-%d %H:%M:%S", ct)
            # s = "%s,%03d" % (t, record.msecs)
            s = str(datetime.datetime.now())
        return s



# coding=utf-8
import pymysql

from pycore.data.entity import config


def get_conn(host=config.get("db", "db_host"),
             port=int(config.get("db", "db_port")),
             user=config.get("db", "db_user"),
             password=config.get("db", "db_pass"),
             db=config.get("db", "db_db"),
             charset=config.get("db", "db_charset"),
             cursorclass=pymysql.cursors.DictCursor):
    return pymysql.connect(host=host, port=port, user=user, password=password, db=db, charset=charset,
                           cursorclass=cursorclass)



# 加密
import base64
from Crypto.Cipher import ARC4

from pycore.utils.stringutils import StringUtils


def new_keys(length):
    return StringUtils.randomStr(length)


# 解密
def rc4_decode(data: str, key: str):
    rc41 = ARC4.new(key.encode('utf-8'))
    return rc41.decrypt(base64.b64decode(data.encode('utf-8'))).decode("utf8")


# 加密
def rc4_encode(data: str, key: str):
    rc41 = ARC4.new(key.encode('utf-8'))
    return base64.b64encode(rc41.encrypt(data.encode('utf-8'))).decode('utf-8').replace('\n', '')  # 加密



import http.client
import traceback
import urllib
import urllib.parse

HTTP_PROXY_HEADER_NAME = ["HTTP_X_FORWARDED_FOR", "X-FORWARDED-FOR", "CLIENTIP", "REMOTE_ADDR"]


class Platform(object):
    UNKNOW = 0
    WINSOWS = 1
    IPHONE = 2
    IPAD = 3
    MAC = 4
    ANDROID = 5
    LINUX = 6


class HttpUtils(object):

    def __init__(self, host):
        self.host = host
        self.reqheaders = {'Content-type': 'application/x-www-form-urlencoded',
                           'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
                           'Host': host,
                           'User-Agent': 'Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1', }

    def get(self, url, param):
        conn = None
        res = None
        try:
            conn = http.client.HTTPConnection(self.host)
            if param is not None:
                data = urllib.parse.urlencode(param)
                conn.request("GET", url + "?" + data, headers=self.reqheaders)
            else:
                conn.request("GET", url, headers=self.reqheaders)
            res = conn.getresponse().read()
        except:
            print(traceback.print_exc())
        finally:
            if conn:
                conn.close()
        return res

    def get_with_ssl(self, url, param):
        conn = None
        res = None
        try:
            conn = http.client.HTTPSConnection(self.host)
            if param is not None:
                data = urllib.parse.urlencode(param)
                conn.request("GET", url + "?" + data, headers=self.reqheaders)
            else:
                conn.request("GET", url, headers=self.reqheaders)
            res = conn.getresponse().read()
        except:
            print(traceback.print_exc())
        finally:
            if conn:
                conn.close()
        return res

    def post(self, url, param):
        conn = None
        res = None
        try:
            conn = http.client.HTTPConnection(self.host)
            if param is not None:
                data = urllib.parse.urlencode(param)
                conn.request('POST', url, data, self.reqheaders)
            else:
                conn.request('POST', url, headers=self.reqheaders)
            res = conn.getresponse().read()
        except:
            print(traceback.print_exc())
        finally:
            if conn:
                conn.close()
        return res

    def post_with_ssl(self, url, param):
        conn = None
        res = None
        try:
            conn = http.client.HTTPSConnection(self.host)
            if param is not None:
                data = urllib.parse.urlencode(param)
                conn.request('POST', url, data, self.reqheaders)
            else:
                conn.request('POST', url, headers=self.reqheaders)
            res = conn.getresponse().read()
        except:
            print(traceback.print_exc())
        finally:
            if conn:
                conn.close()
        return res


def get_client_ip(head):
    return get_client_ip_with_header(head, HTTP_PROXY_HEADER_NAME)


def get_client_ip_with_header(head, headers):
    for name in headers:
        if name in head:
            ip = head[name]
            if len(ip) > 0:
                return ip.split(",")[0]


def get_client_platform(head):
    if "User-Agent" in head:
        user_agent = head["User-Agent"].upper()
        if "WINDOWS" in user_agent:
            return Platform.WINSOWS
        if "IPHONE" in user_agent:
            return Platform.IPHONE
        if "IPAD" in user_agent:
            return Platform.IPAD
        if "MAC" in user_agent:
            return Platform.MAC
        if "ANDROID" in user_agent:
            return Platform.ANDROID
        if "LINUX" in user_agent:
            return Platform.LINUX



# 加密
import base64

from Crypto.Cipher import PKCS1_v1_5
from Crypto.PublicKey import RSA


def new_keys(length):
    private = RSA.generate(length)
    public = private.publickey()
    private_key = private.exportKey()
    public_key = public.exportKey()
    return public_key.decode('utf-8'), private_key.decode('utf-8')


def msg_encrypt(msg: str, pub_key):
    public_key = RSA.importKey(pub_key)
    cipher = PKCS1_v1_5.new(public_key)
    max_msg_length = public_key.size_in_bytes() - 11
    msg_bytes = msg.encode('utf-8')
    secret_bytes = bytes()
    for i in range(0, len(msg_bytes), max_msg_length):
        secret_bytes += base64.b64encode(cipher.encrypt(msg_bytes[i:i + max_msg_length]))
    secret_msg = secret_bytes
    return secret_msg.decode('utf-8')


# 解密
def msg_dencrypt(secret_msg: str, priv_key):
    private_key = RSA.importKey(priv_key)
    cipher = PKCS1_v1_5.new(private_key)
    secret_bytes = base64.b64decode(secret_msg)
    res_msg = bytes()
    max_msg_length = private_key.size_in_bytes()
    for i in range(0, len(secret_bytes), max_msg_length):
        res_msg += cipher.decrypt(secret_bytes[i:i + max_msg_length], b'')
    return res_msg.decode('utf-8')



import time


def string_to_stamp(timestring, tformat):
    return int(time.mktime(time.strptime(timestring, tformat)))


def stamp_to_string(timestamp, tformat):
    time_array = time.localtime(timestamp)
    return time.strftime(tformat, time_array)



# 加密
import base64
import traceback
from Crypto.Cipher import AES

from pycore.utils.stringutils import StringUtils


def new_keys(length):
    return StringUtils.randomStr(length)


# 解密
def aes_decode(data: str, key: str):
    decrypted_text = ''
    try:
        aes = AES.new(key.encode('utf-8'), AES.MODE_ECB)  # 初始化加密器
        decrypted_text = aes.decrypt(base64.b64decode(data.encode('utf-8'))).decode("utf8")  # 解密
        decrypted_text = decrypted_text.rstrip('\0')
    except:
        print(traceback.format_exc())
    return decrypted_text


# 加密
def aes_encode(data: str, key: str):
    count = len(data.encode('utf-8'))
    if (count % 16 != 0):
        add = 16 - (count % 16)
    else:
        add = 0  # 看看你们对接是满16的时候加上16还是0.这里注意
    data = data + ('\0' * add)
    data = data.encode('utf-8')
    aes = AES.new(key.encode('utf-8'), AES.MODE_ECB)  # 初始化加密器
    return base64.b64encode(aes.encrypt(data)).decode('utf-8').replace('\n', '')  # 加密


if __name__ == '__main__':
    msg = '{"type": 2}'
    key = '0s80qxxaxm4o8y4q'
    s = aes_encode(msg, key)
    print(s)
    s = aes_decode(s, key)
    print(s)
